@@ -66,10 +66,9 @@ isis_circuit_new ()
   struct isis_circuit *circuit;
   int i;
 
-  circuit = XMALLOC (MTYPE_ISIS_CIRCUIT, sizeof (struct isis_circuit));
+  circuit = XCALLOC (MTYPE_ISIS_CIRCUIT, sizeof (struct isis_circuit));
   if (circuit)
     {
-      memset (circuit, 0, sizeof (struct isis_circuit));
       /* set default metrics for circuit */
       for (i = 0; i < 2; i++)
 	{
@@ -77,6 +76,7 @@ isis_circuit_new ()
 	  circuit->metrics[i].metric_expense = METRICS_UNSUPPORTED;
 	  circuit->metrics[i].metric_error = METRICS_UNSUPPORTED;
 	  circuit->metrics[i].metric_delay = METRICS_UNSUPPORTED;
+	  circuit->te_metric[i] = DEFAULT_CIRCUIT_METRICS;
 	}
     }
   else
@@ -404,7 +404,9 @@ isis_circuit_if_add (struct isis_circuit *circuit, struct interface *ifp)
     }
   else
     {
-      zlog_warn ("isis_circuit_if_add: unsupported media");
+      /* It's normal in case of loopback etc. */
+      if (isis->debugs & DEBUG_EVENTS)
+	zlog_debug ("isis_circuit_if_add: unsupported media");
     }
 
   for (ALL_LIST_ELEMENTS (ifp->connected, node, nnode, conn))
@@ -618,14 +620,13 @@ isis_interface_config_write (struct vty *vty)
 {
 
   int write = 0;
-  struct listnode *node, *nnode;
-  struct listnode *node2, *nnode2;
+  struct listnode *node, *node2;
   struct interface *ifp;
   struct isis_area *area;
   struct isis_circuit *c;
   int i;
 
-  for (ALL_LIST_ELEMENTS (iflist, node, nnode, ifp))
+  for (ALL_LIST_ELEMENTS_RO (iflist, node, ifp))
   {
     /* IF name */
     vty_out (vty, "interface %s%s", ifp->name, VTY_NEWLINE);
@@ -637,7 +638,7 @@ isis_interface_config_write (struct vty *vty)
 	write++;
       }
     /* ISIS Circuit */
-    for (ALL_LIST_ELEMENTS (isis->area_list, node2, nnode2, area))
+    for (ALL_LIST_ELEMENTS_RO (isis->area_list, node2, area))
     {
       c = circuit_lookup_by_ifp (ifp, area->circuit_list);
       if (c)
@@ -783,12 +784,12 @@ isis_interface_config_write (struct vty *vty)
 		}
 	    }
 	  /* ISIS - Metric */
-	  if (c->metrics[0].metric_default == c->metrics[1].metric_default)
+	  if (c->te_metric[0] == c->te_metric[1])
 	    {
-	      if (c->metrics[0].metric_default != DEFAULT_CIRCUIT_METRICS)
+	      if (c->te_metric[0] != DEFAULT_CIRCUIT_METRICS)
 		{
-		  vty_out (vty, " isis metric %d%s",
-			   c->metrics[0].metric_default, VTY_NEWLINE);
+		  vty_out (vty, " isis metric %d%s", c->te_metric[0],
+			   VTY_NEWLINE);
 		  write++;
 		}
 	    }
@@ -796,11 +797,10 @@ isis_interface_config_write (struct vty *vty)
 	    {
 	      for (i = 0; i < 2; i++)
 		{
-		  if (c->metrics[i].metric_default != DEFAULT_CIRCUIT_METRICS)
+		  if (c->te_metric[i] != DEFAULT_CIRCUIT_METRICS)
 		    {
 		      vty_out (vty, " isis metric %d level-%d%s",
-			       c->metrics[i].metric_default, i + 1,
-			       VTY_NEWLINE);
+			       c->te_metric[i], i + 1, VTY_NEWLINE);
 		      write++;
 		    }
 		}
@@ -842,7 +842,8 @@ DEFUN (ip_router_isis,
       if (c->ipv6_router == 0)
 	{
 #endif /* HAVE_IPV6 */
-	  vty_out (vty, "ISIS circuit is already defined%s", VTY_NEWLINE);
+	  /* FIXME: Find the way to warn only vty users. */
+	  /* vty_out (vty, "ISIS circuit is already defined%s", VTY_NEWLINE); */
 	  return CMD_WARNING;
 #ifdef HAVE_IPV6
 	}
@@ -852,7 +853,8 @@ DEFUN (ip_router_isis,
   /* this is here for ciscopability */
   if (!area)
     {
-      vty_out (vty, "Can't find ISIS instance %s", VTY_NEWLINE);
+      /* FIXME: Find the way to warn only vty users. */
+      /* vty_out (vty, "Can't find ISIS instance %s", VTY_NEWLINE); */
       return CMD_WARNING;
     }
 
@@ -888,7 +890,7 @@ DEFUN (no_ip_router_isis,
   struct isis_circuit *circuit = NULL;
   struct interface *ifp;
   struct isis_area *area;
-  struct listnode *node, *nnode;
+  struct listnode *node;
 
   ifp = (struct interface *) vty->index;
   assert (ifp);
@@ -899,7 +901,7 @@ DEFUN (no_ip_router_isis,
       vty_out (vty, "Can't find ISIS instance %s", VTY_NEWLINE);
       return CMD_WARNING;
     }
-  for (ALL_LIST_ELEMENTS (area->circuit_list, node, nnode, circuit))
+  for (ALL_LIST_ELEMENTS_RO (area->circuit_list, node, circuit))
     if (circuit->interface == ifp)
       break;
   if (!circuit)
@@ -1231,7 +1233,7 @@ ALIAS (no_isis_priority_l2,
 /* Metric command */
   DEFUN (isis_metric,
        isis_metric_cmd,
-       "isis metric <0-63>",
+       "isis metric <0-16777215>",
        "IS-IS commands\n"
        "Set default metric for circuit\n"
        "Default metric value\n")
@@ -1250,6 +1252,12 @@ ALIAS (no_isis_priority_l2,
 
   met = atoi (argv[0]);
 
+  circuit->te_metric[0] = met;
+  circuit->te_metric[1] = met;
+
+  if (met > 63)
+    met = 63;
+
   circuit->metrics[0].metric_default = met;
   circuit->metrics[1].metric_default = met;
 
@@ -1274,6 +1282,8 @@ DEFUN (no_isis_metric,
     }
   assert (circuit);
 
+  circuit->te_metric[0] = DEFAULT_CIRCUIT_METRICS;
+  circuit->te_metric[1] = DEFAULT_CIRCUIT_METRICS;
   circuit->metrics[0].metric_default = DEFAULT_CIRCUIT_METRICS;
   circuit->metrics[1].metric_default = DEFAULT_CIRCUIT_METRICS;
 
@@ -1282,14 +1292,14 @@ DEFUN (no_isis_metric,
 
 ALIAS (no_isis_metric,
        no_isis_metric_arg_cmd,
-       "no isis metric <0-127>",
+       "no isis metric <0-16777215>",
        NO_STR
        "IS-IS commands\n"
        "Set default metric for circuit\n"
        "Default metric value\n")
 
 /* end of metrics */
-  DEFUN (isis_hello_interval,
+DEFUN (isis_hello_interval,
        isis_hello_interval_cmd,
        "isis hello-interval (<1-65535>|minimal)",
        "IS-IS commands\n"
