@@ -125,11 +125,8 @@ isis_event_system_type_change (struct isis_area *area, int newtype)
     }
 
   area->is_type = newtype;
-  for (node = listhead (area->circuit_list); node; nextnode (node))
-    {
-      circuit = getdata (node);
-      isis_event_circuit_type_change (circuit, newtype);
-    }
+  for (ALL_LIST_ELEMENTS_RO (area->circuit_list, node, circuit))
+    isis_event_circuit_type_change (circuit, newtype);
 
   spftree_area_init (area);
   lsp_regenerate_schedule (area);
@@ -143,7 +140,7 @@ isis_event_area_addr_change (struct isis_area *area)
 
 }
 
-void
+static void
 circuit_commence_level (struct isis_circuit *circuit, int level)
 {
   if (level == 1)
@@ -186,7 +183,7 @@ circuit_commence_level (struct isis_circuit *circuit, int level)
   return;
 }
 
-void
+static void
 circuit_resign_level (struct isis_circuit *circuit, int level)
 {
   int idx = level - 1;
@@ -198,6 +195,7 @@ circuit_resign_level (struct isis_circuit *circuit, int level)
     {
       THREAD_TIMER_OFF (circuit->u.bc.t_send_lan_hello[idx]);
       THREAD_TIMER_OFF (circuit->u.bc.t_run_dr[idx]);
+      THREAD_TIMER_OFF (circuit->u.bc.t_refresh_pseudo_lsp[idx]);
       circuit->u.bc.run_dr_elect[idx] = 0;
     }
 
@@ -286,8 +284,9 @@ isis_event_adjacency_state_change (struct isis_adjacency *adj, int newstate)
   if (!adj || !adj->circuit || !adj->circuit->area)
     return;
 
-  zlog_debug ("ISIS-Evt (%s) Adjacency State change",
-	      adj->circuit->area->area_tag);
+  if (isis->debugs & DEBUG_EVENTS)
+    zlog_debug ("ISIS-Evt (%s) Adjacency State change",
+		adj->circuit->area->area_tag);
 
   /* LSP generation again */
   lsp_regenerate_schedule (adj->circuit->area);
@@ -307,8 +306,8 @@ isis_event_dis_status_change (struct thread *thread)
   /* invalid arguments */
   if (!circuit || !circuit->area)
     return 0;
-
-  zlog_debug ("ISIS-Evt (%s) DIS status change", circuit->area->area_tag);
+  if (isis->debugs & DEBUG_EVENTS)
+    zlog_debug ("ISIS-Evt (%s) DIS status change", circuit->area->area_tag);
 
   /* LSP generation again */
   lsp_regenerate_schedule (circuit->area);
@@ -319,8 +318,9 @@ isis_event_dis_status_change (struct thread *thread)
 void
 isis_event_auth_failure (char *area_tag, const char *error_string, u_char *sysid)
 {
-  zlog_debug ("ISIS-Evt (%s) Authentication failure %s from %s",
-	      area_tag, error_string, sysid_print (sysid));
+  if (isis->debugs & DEBUG_EVENTS)
+    zlog_debug ("ISIS-Evt (%s) Authentication failure %s from %s",
+		area_tag, error_string, sysid_print (sysid));
 
   return;
 }
