@@ -43,9 +43,10 @@
 #include "ospfd/ospf_packet.h"
 #include "ospfd/ospf_flood.h"
 #include "ospfd/ospf_abr.h"
+#include "ospfd/ospf_snmp.h"
 
 /* elect DR and BDR. Refer to RFC2319 section 9.4 */
-struct ospf_neighbor *
+static struct ospf_neighbor *
 ospf_dr_election_sub (struct list *routers)
 {
   struct listnode *node;
@@ -70,7 +71,7 @@ ospf_dr_election_sub (struct list *routers)
   return max;
 }
 
-struct ospf_neighbor *
+static struct ospf_neighbor *
 ospf_elect_dr (struct ospf_interface *oi, struct list *el_list)
 {
   struct list *dr_list;
@@ -99,19 +100,16 @@ ospf_elect_dr (struct ospf_interface *oi, struct list *el_list)
 
   /* Set DR to interface. */
   if (dr)
-    {
-      DR (oi) = dr->address.u.prefix4;
-      dr->d_router = dr->address.u.prefix4;
-    }
+    DR (oi) = dr->address.u.prefix4;
   else
-      DR (oi).s_addr = 0;
+    DR (oi).s_addr = 0;
 
   list_delete (dr_list);
 
   return dr;
 }
 
-struct ospf_neighbor *
+static struct ospf_neighbor *
 ospf_elect_bdr (struct ospf_interface *oi, struct list *el_list)
 {
   struct list *bdr_list, *no_dr_list;
@@ -143,10 +141,7 @@ ospf_elect_bdr (struct ospf_interface *oi, struct list *el_list)
 
   /* Set BDR to interface. */
   if (bdr)
-    {
-      BDR (oi) = bdr->address.u.prefix4;
-      bdr->bd_router = bdr->address.u.prefix4;
-    }
+    BDR (oi) = bdr->address.u.prefix4;
   else
     BDR (oi).s_addr = 0;
 
@@ -156,7 +151,7 @@ ospf_elect_bdr (struct ospf_interface *oi, struct list *el_list)
   return bdr;
 }
 
-int
+static int
 ospf_ism_state (struct ospf_interface *oi)
 {
   if (IPV4_ADDR_SAME (&DR (oi), &oi->address->u.prefix4))
@@ -167,7 +162,7 @@ ospf_ism_state (struct ospf_interface *oi)
     return ISM_DROther;
 }
 
-void
+static void
 ospf_dr_eligible_routers (struct route_table *nbrs, struct list *el_list)
 {
   struct route_node *rn;
@@ -185,7 +180,7 @@ ospf_dr_eligible_routers (struct route_table *nbrs, struct list *el_list)
 }
 
 /* Generate AdjOK? NSM event. */
-void
+static void
 ospf_dr_change (struct ospf *ospf, struct route_table *nbrs)
 {
   struct route_node *rn;
@@ -202,7 +197,7 @@ ospf_dr_change (struct ospf *ospf, struct route_table *nbrs)
 	    OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_AdjOK);
 }
 
-int
+static int
 ospf_dr_election (struct ospf_interface *oi)
 {
   struct in_addr old_dr, old_bdr;
@@ -268,13 +263,12 @@ ospf_hello_timer (struct thread *thread)
   ospf_hello_send (oi);
 
   /* Hello timer set. */
-  OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer,
-		     OSPF_IF_PARAM (oi, v_hello));
-
+  OSPF_HELLO_TIMER_ON (oi);
+  
   return 0;
 }
 
-int
+static int
 ospf_wait_timer (struct thread *thread)
 {
   struct ospf_interface *oi;
@@ -294,7 +288,7 @@ ospf_wait_timer (struct thread *thread)
 /* Hook function called after ospf ISM event is occured. And vty's
    network command invoke this function after making interface
    structure. */
-void
+static void
 ism_timer_set (struct ospf_interface *oi)
 {
   switch (oi->state)
@@ -318,7 +312,7 @@ ism_timer_set (struct ospf_interface *oi)
       /* The router is trying to determine the identity of DRouter and
 	 BDRouter. The router begin to receive and send Hello Packets. */
       /* send first hello immediately */
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer, 1);
+      OSPF_ISM_TIMER_MSEC_ON (oi->t_hello, ospf_hello_timer, 1);
       OSPF_ISM_TIMER_ON (oi->t_wait, ospf_wait_timer,
 			 OSPF_IF_PARAM (oi, v_wait));
       OSPF_ISM_TIMER_OFF (oi->t_ls_ack);
@@ -328,8 +322,7 @@ ism_timer_set (struct ospf_interface *oi)
 	 virtual link. The router attempts to form an adjacency with
 	 neighboring router. Hello packets are also sent. */
       /* send first hello immediately */
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer, 1);
-      
+      OSPF_ISM_TIMER_MSEC_ON (oi->t_hello, ospf_hello_timer, 1);      
       OSPF_ISM_TIMER_OFF (oi->t_wait);
       OSPF_ISM_TIMER_ON (oi->t_ls_ack, ospf_ls_ack_timer, oi->v_ls_ack);
       break;
@@ -337,37 +330,28 @@ ism_timer_set (struct ospf_interface *oi)
       /* The network type of the interface is broadcast or NBMA network,
 	 and the router itself is neither Designated Router nor
 	 Backup Designated Router. */
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer,
-			 OSPF_IF_PARAM (oi, v_hello));
+      OSPF_HELLO_TIMER_ON (oi);
       OSPF_ISM_TIMER_OFF (oi->t_wait);
       OSPF_ISM_TIMER_ON (oi->t_ls_ack, ospf_ls_ack_timer, oi->v_ls_ack);
       break;
     case ISM_Backup:
       /* The network type of the interface is broadcast os NBMA network,
 	 and the router is Backup Designated Router. */
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer,
-			 OSPF_IF_PARAM (oi, v_hello));
+      OSPF_HELLO_TIMER_ON (oi);
       OSPF_ISM_TIMER_OFF (oi->t_wait);
       OSPF_ISM_TIMER_ON (oi->t_ls_ack, ospf_ls_ack_timer, oi->v_ls_ack);
       break;
     case ISM_DR:
       /* The network type of the interface is broadcast or NBMA network,
 	 and the router is Designated Router. */
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer,
-			 OSPF_IF_PARAM (oi, v_hello));
+      OSPF_HELLO_TIMER_ON (oi);
       OSPF_ISM_TIMER_OFF (oi->t_wait);
       OSPF_ISM_TIMER_ON (oi->t_ls_ack, ospf_ls_ack_timer, oi->v_ls_ack);
       break;
     }
 }
 
-int
-ism_stop (struct ospf_interface *oi)
-{
-  return 0;
-}
-
-int
+static int
 ism_interface_up (struct ospf_interface *oi)
 {
   int next_state = 0;
@@ -393,7 +377,7 @@ ism_interface_up (struct ospf_interface *oi)
   return next_state;
 }
 
-int
+static int
 ism_loop_ind (struct ospf_interface *oi)
 {
   int ret = 0;
@@ -405,7 +389,7 @@ ism_loop_ind (struct ospf_interface *oi)
 }
 
 /* Interface down event handler. */
-int
+static int
 ism_interface_down (struct ospf_interface *oi)
 {
   ospf_if_cleanup (oi);
@@ -413,25 +397,25 @@ ism_interface_down (struct ospf_interface *oi)
 }
 
 
-int
+static int
 ism_backup_seen (struct ospf_interface *oi)
 {
   return ospf_dr_election (oi);
 }
 
-int
+static int
 ism_wait_timer (struct ospf_interface *oi)
 {
   return ospf_dr_election (oi);
 }
 
-int
+static int
 ism_neighbor_change (struct ospf_interface *oi)
 {
   return ospf_dr_election (oi);
 }
 
-int
+static int
 ism_ignore (struct ospf_interface *oi)
 {
   if (IS_DEBUG_OSPF (ism, ISM_EVENTS))
@@ -442,7 +426,7 @@ ism_ignore (struct ospf_interface *oi)
 
 /* Interface State Machine */
 struct {
-  int (*func) ();
+  int (*func) (struct ospf_interface *);
   int next_state;
 } ISM [OSPF_ISM_STATE_MAX][OSPF_ISM_EVENT_MAX] =
 {
@@ -548,7 +532,7 @@ const static char *ospf_ism_event_str[] =
   "InterfaceDown",
 };
 
-void
+static void
 ism_change_state (struct ospf_interface *oi, int state)
 {
   int old_state;
@@ -564,6 +548,20 @@ ism_change_state (struct ospf_interface *oi, int state)
   oi->state = state;
   oi->state_change++;
 
+#ifdef HAVE_SNMP
+  /* Terminal state or regression */ 
+  if ((state == ISM_DR) || (state == ISM_Backup) || (state == ISM_DROther) ||
+      (state == ISM_PointToPoint) || (state < old_state))
+    {
+      /* ospfVirtIfStateChange */
+      if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
+        ospfTrapVirtIfStateChange (oi);
+      /* ospfIfStateChange */
+      else
+        ospfTrapIfStateChange (oi);
+    }
+#endif
+
   /* Set multicast memberships appropriately for new state. */
   ospf_if_set_multicast(oi);
 
