@@ -47,12 +47,13 @@
 #include "ospfd/ospf_dump.h"
 #include "ospfd/ospf_flood.h"
 #include "ospfd/ospf_abr.h"
+#include "ospfd/ospf_snmp.h"
 
 void nsm_reset_nbr (struct ospf_neighbor *);
 
 
 /* OSPF NSM Timer functions. */
-int
+static int
 ospf_inactivity_timer (struct thread *thread)
 {
   struct ospf_neighbor *nbr;
@@ -69,7 +70,7 @@ ospf_inactivity_timer (struct thread *thread)
   return 0;
 }
 
-int
+static int
 ospf_db_desc_timer (struct thread *thread)
 {
   struct ospf_interface *oi;
@@ -96,7 +97,7 @@ ospf_db_desc_timer (struct thread *thread)
 
 /* Hook function called after ospf NSM event is occured. */
 
-void
+static void
 nsm_timer_set (struct ospf_neighbor *nbr)
 {
   switch (nbr->state)
@@ -140,7 +141,7 @@ nsm_timer_set (struct ospf_neighbor *nbr)
 
 
 /* OSPF NSM functions. */
-int
+static int
 nsm_ignore (struct ospf_neighbor *nbr)
 {
   if (IS_DEBUG_OSPF (nsm, NSM_EVENTS))
@@ -150,7 +151,7 @@ nsm_ignore (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_hello_received (struct ospf_neighbor *nbr)
 {
   /* Start or Restart Inactivity Timer. */
@@ -165,7 +166,7 @@ nsm_hello_received (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_start (struct ospf_neighbor *nbr)
 {
 
@@ -182,7 +183,7 @@ nsm_start (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_twoway_received (struct ospf_neighbor *nbr)
 {
   struct ospf_interface *oi;
@@ -221,7 +222,7 @@ ospf_db_summary_isempty (struct ospf_neighbor *nbr)
   return ospf_lsdb_isempty (&nbr->db_sum);
 }
 
-int
+static int
 ospf_db_summary_add (struct ospf_neighbor *nbr, struct ospf_lsa *lsa)
 {
 #ifdef HAVE_OPAQUE_LSA
@@ -282,7 +283,7 @@ ospf_db_summary_clear (struct ospf_neighbor *nbr)
    AS-external-LSAs are omitted from a virtual neighbor's Database
    summary list.  AS-external-LSAs are omitted from the Database
    summary list if the area has been configured as a stub. */
-int
+static int
 nsm_negotiation_done (struct ospf_neighbor *nbr)
 {
   struct ospf_area *area = nbr->oi->area;
@@ -312,7 +313,7 @@ nsm_negotiation_done (struct ospf_neighbor *nbr)
   if (CHECK_FLAG (nbr->options, OSPF_OPTION_NP))
     {
       LSDB_LOOP (NSSA_LSDB (area), rn, lsa)
-	 ospf_db_summary_add (nbr, lsa);
+	ospf_db_summary_add (nbr, lsa);
     }
 
   if (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK
@@ -331,7 +332,7 @@ nsm_negotiation_done (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_exchange_done (struct ospf_neighbor *nbr)
 {
   if (ospf_ls_request_isempty (nbr))
@@ -346,7 +347,7 @@ nsm_exchange_done (struct ospf_neighbor *nbr)
   return NSM_Loading;
 }
 
-int
+static int
 nsm_bad_ls_req (struct ospf_neighbor *nbr)
 {
   /* Clear neighbor. */
@@ -355,7 +356,7 @@ nsm_bad_ls_req (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_adj_ok (struct ospf_neighbor *nbr)
 {
   struct ospf_interface *oi;
@@ -388,7 +389,7 @@ nsm_adj_ok (struct ospf_neighbor *nbr)
   return next_state;
 }
 
-int
+static int
 nsm_seq_number_mismatch (struct ospf_neighbor *nbr)
 {
   /* Clear neighbor. */
@@ -397,7 +398,7 @@ nsm_seq_number_mismatch (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_oneway_received (struct ospf_neighbor *nbr)
 {
   /* Clear neighbor. */
@@ -433,7 +434,7 @@ nsm_reset_nbr (struct ospf_neighbor *nbr)
 #endif /* HAVE_OPAQUE_LSA */
 }
 
-int
+static int
 nsm_kill_nbr (struct ospf_neighbor *nbr)
 {
   /* call it here because we cannot call it from ospf_nsm_event */
@@ -465,7 +466,7 @@ nsm_kill_nbr (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_inactivity_timer (struct ospf_neighbor *nbr)
 {
   /* Kill neighbor. */
@@ -474,7 +475,7 @@ nsm_inactivity_timer (struct ospf_neighbor *nbr)
   return 0;
 }
 
-int
+static int
 nsm_ll_down (struct ospf_neighbor *nbr)
 {
   /* Reset neighbor. */
@@ -488,7 +489,7 @@ nsm_ll_down (struct ospf_neighbor *nbr)
 
 /* Neighbor State Machine */
 struct {
-  int (*func) ();
+  int (*func) (struct ospf_neighbor *);
   int next_state;
 } NSM [OSPF_NSM_STATE_MAX][OSPF_NSM_EVENT_MAX] =
 {
@@ -692,6 +693,22 @@ nsm_change_state (struct ospf_neighbor *nbr, int state)
   if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
     vl_area = ospf_area_lookup_by_area_id (oi->ospf, oi->vl_data->vl_area_id);
   
+#ifdef HAVE_SNMP
+  /* Terminal state or regression */ 
+  if ((state == NSM_Full) || (state == NSM_TwoWay) || (state < old_state))
+    {
+      /* ospfVirtNbrStateChange */
+      if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
+        ospfTrapVirtNbrStateChange(nbr);
+      /* ospfNbrStateChange trap  */
+      else
+        /* To/From FULL, only managed by DR */
+        if (((state != NSM_Full) && (old_state != NSM_Full)) ||
+            (oi->state == ISM_DR))
+          ospfTrapNbrStateChange(nbr);
+    }
+#endif
+
   /* One of the neighboring routers changes to/from the FULL state. */
   if ((old_state != NSM_Full && state == NSM_Full) ||
       (old_state == NSM_Full && state != NSM_Full))
@@ -809,7 +826,7 @@ nsm_change_state (struct ospf_neighbor *nbr, int state)
   if (state == NSM_Init)
     {
       OSPF_ISM_TIMER_OFF (oi->t_hello);
-      OSPF_ISM_TIMER_ON (oi->t_hello, ospf_hello_timer, 1);
+      OSPF_ISM_TIMER_MSEC_ON (oi->t_hello, ospf_hello_timer, 1);
     }
 
   /* Preserve old status? */
