@@ -74,7 +74,7 @@ ospf_path_new ()
   return new;
 }
 
-struct ospf_path *
+static struct ospf_path *
 ospf_path_dup (struct ospf_path *path)
 {
   struct ospf_path *new;
@@ -177,7 +177,7 @@ ospf_route_match_same (struct route_table *rt, struct prefix_ipv4 *prefix,
 /* delete routes generated from AS-External routes if there is a inter/intra
  * area route
  */
-void 
+static void 
 ospf_route_delete_same_ext(struct route_table *external_routes,
                      struct route_table *routes)
 {
@@ -208,7 +208,7 @@ ospf_route_delete_same_ext(struct route_table *external_routes,
 }
 
 /* rt: Old, cmprt: New */
-void
+static void
 ospf_route_delete_uniq (struct route_table *rt, struct route_table *cmprt)
 {
   struct route_node *rn;
@@ -270,7 +270,7 @@ ospf_route_install (struct ospf *ospf, struct route_table *rt)
       }
 }
 
-void
+static void
 ospf_intra_route_add (struct route_table *rt, struct vertex *v,
 		      struct ospf_area *area)
 {
@@ -278,7 +278,7 @@ ospf_intra_route_add (struct route_table *rt, struct vertex *v,
   struct ospf_route *or;
   struct prefix_ipv4 p;
   struct ospf_path *path;
-  struct vertex_nexthop *nexthop;
+  struct vertex_parent *parent;
   struct listnode *node, *nnode;
 
   p.family = AF_INET;
@@ -306,10 +306,10 @@ ospf_intra_route_add (struct route_table *rt, struct vertex *v,
     {
       or->type = OSPF_DESTINATION_NETWORK;
 
-      for (ALL_LIST_ELEMENTS (v->nexthop, node, nnode, nexthop))
+      for (ALL_LIST_ELEMENTS (v->parents, node, nnode, parent))
         {
           path = ospf_path_new ();
-          path->nexthop = nexthop->router;
+          path->nexthop = parent->nexthop->router;
           listnode_add (or->paths, path);
         }
     }
@@ -690,26 +690,11 @@ ospf_route_table_dump (struct route_table *rt)
   zlog_debug ("========================================");
 }
 
-void
-ospf_terminate ()
-{
-  struct ospf *ospf;
-  struct listnode *node, *nnode;
-
-  for (ALL_LIST_ELEMENTS (om->ospf, node, nnode, ospf))
-    {
-      if (ospf->new_table)
-	ospf_route_delete (ospf->new_table);
-      if (ospf->old_external_route)
-	ospf_route_delete (ospf->old_external_route);
-    }
-}
-
 /* This is 16.4.1 implementation.
    o Intra-area paths using non-backbone areas are always the most preferred.
    o The other paths, intra-area backbone paths and inter-area paths,
      are of equal preference. */
-int
+static int
 ospf_asbr_route_cmp (struct ospf *ospf, struct ospf_route *r1,
 		     struct ospf_route *r2)
 {
@@ -778,7 +763,7 @@ ospf_route_cmp (struct ospf *ospf, struct ospf_route *r1,
   return (r1->cost - r2->cost);
 }
 
-int
+static int
 ospf_path_exist (struct list *plist, struct in_addr nexthop,
 		 struct ospf_interface *oi)
 {
@@ -799,11 +784,14 @@ ospf_route_copy_nexthops_from_vertex (struct ospf_route *to,
   struct listnode *node;
   struct ospf_path *path;
   struct vertex_nexthop *nexthop;
+  struct vertex_parent *vp;
 
   assert (to->paths);
 
-  for (ALL_LIST_ELEMENTS_RO (v->nexthop, node, nexthop))
+  for (ALL_LIST_ELEMENTS_RO (v->parents, node, vp))
     {
+      nexthop = vp->nexthop;
+      
       if (nexthop->oi != NULL) 
 	{
 	  if (! ospf_path_exist (to->paths, nexthop->router, nexthop->oi))
