@@ -57,7 +57,7 @@ extern struct thread_master *master;
 struct in_addr router_id_zebra;
 
 /* Router-id update message from zebra. */
-int
+static int
 ospf_router_id_update_zebra (int command, struct zclient *zclient,
 			     zebra_size_t length)
 {
@@ -78,7 +78,7 @@ ospf_router_id_update_zebra (int command, struct zclient *zclient,
 }
 
 /* Inteface addition message from zebra. */
-int
+static int
 ospf_interface_add (int command, struct zclient *zclient, zebra_size_t length)
 {
   struct interface *ifp;
@@ -109,7 +109,7 @@ ospf_interface_add (int command, struct zclient *zclient, zebra_size_t length)
   return 0;
 }
 
-int
+static int
 ospf_interface_delete (int command, struct zclient *zclient,
                        zebra_size_t length)
 {
@@ -158,7 +158,7 @@ zebra_interface_if_lookup (struct stream *s)
 			       strnlen(ifname_tmp, INTERFACE_NAMSIZ));
 }
 
-int
+static int
 ospf_interface_state_up (int command, struct zclient *zclient,
                          zebra_size_t length)
 {
@@ -220,7 +220,7 @@ ospf_interface_state_up (int command, struct zclient *zclient,
   return 0;
 }
 
-int
+static int
 ospf_interface_state_down (int command, struct zclient *zclient,
                            zebra_size_t length)
 {
@@ -246,7 +246,7 @@ ospf_interface_state_down (int command, struct zclient *zclient,
   return 0;
 }
 
-int
+static int
 ospf_interface_address_add (int command, struct zclient *zclient,
                             zebra_size_t length)
 {
@@ -269,7 +269,7 @@ ospf_interface_address_add (int command, struct zclient *zclient,
   return 0;
 }
 
-int
+static int
 ospf_interface_address_delete (int command, struct zclient *zclient,
                                zebra_size_t length)
 {
@@ -527,7 +527,7 @@ ospf_redistribute_set (struct ospf *ospf, int type, int mtype, int mvalue)
 
       if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
         zlog_debug ("Redistribute[%s]: Refresh  Type[%d], Metric[%d]",
-                   LOOKUP (ospf_redistributed_proto, type),
+                   ospf_redist_string(type),
                    metric_type (ospf, type), metric_value (ospf, type));
 
       return CMD_SUCCESS;
@@ -540,7 +540,7 @@ ospf_redistribute_set (struct ospf *ospf, int type, int mtype, int mvalue)
 
   if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
     zlog_debug ("Redistribute[%s]: Start  Type[%d], Metric[%d]",
-               LOOKUP (ospf_redistributed_proto, type),
+               ospf_redist_string(type),
                metric_type (ospf, type), metric_value (ospf, type));
 
   ospf_asbr_status_update (ospf, ++ospf->redistribute);
@@ -561,7 +561,7 @@ ospf_redistribute_unset (struct ospf *ospf, int type)
 
   if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
     zlog_debug ("Redistribute[%s]: Stop",
-               LOOKUP (ospf_redistributed_proto, type));
+               ospf_redist_string(type));
 
   ospf->dmetric[type].type = -1;
   ospf->dmetric[type].value = -1;
@@ -597,7 +597,7 @@ ospf_redistribute_default_set (struct ospf *ospf, int originate,
 
       if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
         zlog_debug ("Redistribute[%s]: Refresh  Type[%d], Metric[%d]",
-                   LOOKUP (ospf_redistributed_proto, DEFAULT_ROUTE),
+                   ospf_redist_string(DEFAULT_ROUTE),
                    metric_type (ospf, DEFAULT_ROUTE),
                    metric_value (ospf, DEFAULT_ROUTE));
       return CMD_SUCCESS;
@@ -645,7 +645,7 @@ ospf_redistribute_default_unset (struct ospf *ospf)
   return CMD_SUCCESS;
 }
 
-int
+static int
 ospf_external_lsa_originate_check (struct ospf *ospf,
                                    struct external_info *ei)
 {
@@ -713,7 +713,7 @@ ospf_redistribute_check (struct ospf *ospf,
         {
           if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
             zlog_debug ("Redistribute[%s]: %s/%d filtered by ditribute-list.",
-                       LOOKUP (ospf_redistributed_proto, type),
+                       ospf_redist_string(type),
                        inet_ntoa (p->prefix), p->prefixlen);
           return 0;
         }
@@ -734,7 +734,7 @@ ospf_redistribute_check (struct ospf *ospf,
           ei->route_map_set = save_values;
           if (IS_DEBUG_OSPF (zebra, ZEBRA_REDISTRIBUTE))
             zlog_debug ("Redistribute[%s]: %s/%d filtered by route-map.",
-                       LOOKUP (ospf_redistributed_proto, type),
+                       ospf_redist_string(type),
                        inet_ntoa (p->prefix), p->prefixlen);
           return 0;
         }
@@ -770,7 +770,7 @@ ospf_routemap_unset (struct ospf *ospf, int type)
 }
 
 /* Zebra route add and delete treatment. */
-int
+static int
 ospf_zebra_read_ipv4 (int command, struct zclient *zclient,
                       zebra_size_t length)
 {
@@ -869,7 +869,7 @@ ospf_zebra_read_ipv4 (int command, struct zclient *zclient,
       if (is_prefix_default (&p))
         ospf_external_lsa_refresh_default (ospf);
       else
-        ospf_external_lsa_flush (ospf, api.type, &p, ifindex, nexthop);
+        ospf_external_lsa_flush (ospf, api.type, &p, ifindex /*, nexthop */);
     }
 
   return 0;
@@ -916,18 +916,18 @@ ospf_distribute_list_out_unset (struct ospf *ospf, int type, const char *name)
 }
 
 /* distribute-list update timer. */
-int
+static int
 ospf_distribute_list_update_timer (struct thread *thread)
 {
   struct route_node *rn;
   struct external_info *ei;
   struct route_table *rt;
   struct ospf_lsa *lsa;
-  int type;
+  intptr_t type;
   struct ospf *ospf;
 
-  type = (int) THREAD_ARG (thread);
-  assert (type < ZEBRA_ROUTE_MAX);
+  type = (intptr_t)THREAD_ARG (thread);
+  assert (type <= ZEBRA_ROUTE_MAX);
   
   rt = EXTERNAL_INFO (type);
 
@@ -977,7 +977,7 @@ ospf_distribute_list_update (struct ospf *ospf, int type)
 }
 
 /* If access-list is updated, apply some check. */
-void
+static void
 ospf_filter_update (struct access_list *access)
 {
   struct ospf *ospf;
@@ -992,7 +992,7 @@ ospf_filter_update (struct access_list *access)
     return;
 
   /* Update distribute-list, and apply filter. */
-  for (type = 0; type < ZEBRA_ROUTE_MAX; type++)
+  for (type = 0; type <= ZEBRA_ROUTE_MAX; type++)
     {
       if (ROUTEMAP (ospf, type) != NULL)
         {
@@ -1001,6 +1001,10 @@ ospf_filter_update (struct access_list *access)
           continue;
         }
 
+      /* There is place for route-map for default-information (ZEBRA_ROUTE_MAX),
+       * but no distribute list. */
+      if (type == ZEBRA_ROUTE_MAX)
+	break;
 
       if (DISTRIBUTE_NAME (ospf, type))
         {
@@ -1061,7 +1065,7 @@ ospf_prefix_list_update (struct prefix_list *plist)
   /* Update all route-maps which are used as redistribution filters.
    * They might use prefix-list.
    */
-  for (type = 0; type < ZEBRA_ROUTE_MAX; type++)
+  for (type = 0; type <= ZEBRA_ROUTE_MAX; type++)
     {
       if (ROUTEMAP (ospf, type) != NULL)
         {
@@ -1098,8 +1102,8 @@ ospf_prefix_list_update (struct prefix_list *plist)
     ospf_schedule_abr_task (ospf);
 }
 
-struct ospf_distance *
-ospf_distance_new ()
+static struct ospf_distance *
+ospf_distance_new (void)
 {
   struct ospf_distance *new;
   new = XMALLOC (MTYPE_OSPF_DISTANCE, sizeof (struct ospf_distance));
@@ -1107,7 +1111,7 @@ ospf_distance_new ()
   return new;
 }
 
-void
+static void
 ospf_distance_free (struct ospf_distance *odistance)
 {
   XFREE (MTYPE_OSPF_DISTANCE, odistance);
