@@ -88,12 +88,15 @@
 #define OSPF_AUTH_CMD_NOTSEEN              -2
 
 /* OSPF SPF timer values. */
-#define OSPF_SPF_DELAY_DEFAULT              1
-#define OSPF_SPF_HOLDTIME_DEFAULT           1
+#define OSPF_SPF_DELAY_DEFAULT              200
+#define OSPF_SPF_HOLDTIME_DEFAULT           1000
+#define OSPF_SPF_MAX_HOLDTIME_DEFAULT	    10000
 
 /* OSPF interface default values. */
 #define OSPF_OUTPUT_COST_DEFAULT           10
+#define OSPF_OUTPUT_COST_INFINITE	   UINT16_MAX
 #define OSPF_ROUTER_DEAD_INTERVAL_DEFAULT  40
+#define OSPF_ROUTER_DEAD_INTERVAL_MINIMAL   1
 #define OSPF_HELLO_INTERVAL_DEFAULT        10
 #define OSPF_ROUTER_PRIORITY_DEFAULT        1
 #define OSPF_RETRANSMIT_INTERVAL_DEFAULT    5
@@ -105,6 +108,9 @@
 #define OSPF_POLL_INTERVAL_DEFAULT         60
 #define OSPF_NEIGHBOR_PRIORITY_DEFAULT      0
 
+#define OSPF_MTU_IGNORE_DEFAULT             0
+#define OSPF_FAST_HELLO_DEFAULT             0
+
 /* OSPF options. */
 #define OSPF_OPTION_T                    0x01  /* TOS. */
 #define OSPF_OPTION_E                    0x02
@@ -147,6 +153,7 @@ struct ospf_master
 
   /* Various OSPF global configuration. */
   u_char options;
+#define OSPF_MASTER_SHUTDOWN (1 << 0) /* flag for ospf_finish / graceful-shutdown */  
 };
 
 /* OSPF instance structure. */
@@ -168,6 +175,7 @@ struct ospf
 #define OSPF_ABR_IBM            2
 #define OSPF_ABR_CISCO          3
 #define OSPF_ABR_SHORTCUT       4
+#define OSPF_ABR_DEFAULT	OSPF_ABR_CISCO
 
   /* NSSA ABR */
   u_char anyNSSA;		/* Bump for every NSSA attached. */
@@ -186,8 +194,18 @@ struct ospf
 #define OPAQUE_BLOCK_TYPE_11_LSA_BIT	(1 << 3)
 #endif /* HAVE_OPAQUE_LSA */
 
-  int spf_delay;			/* SPF delay time. */
-  int spf_holdtime;			/* SPF hold time. */
+  /* RFC3137 stub router. Configured time to stay stub / max-metric */
+  unsigned int stub_router_startup_time;	/* seconds */
+  unsigned int stub_router_shutdown_time;	/* seconds */
+#define OSPF_STUB_ROUTER_UNCONFIGURED	  0
+#define OSPF_STUB_ROUTER_SHUTDOWN_DEFAULT 2
+
+  /* SPF parameters */
+  unsigned int spf_delay;		/* SPF delay time. */
+  unsigned int spf_holdtime;		/* SPF hold time. */
+  unsigned int spf_max_holdtime;	/* SPF maximum-holdtime */
+  unsigned int spf_hold_multiplier;	/* Adaptive multiplier for hold time */
+  
   int default_originate;		/* Default information originate. */
 #define DEFAULT_ORIGINATE_NONE		0
 #define DEFAULT_ORIGINATE_ZEBRA		1
@@ -226,7 +244,7 @@ struct ospf
 					   prefix is LSA's adv. network*/
 
   /* Time stamps. */
-  time_t ts_spf;			/* SPF calculation time stamp. */
+  struct timeval ts_spf;		/* SPF calculation time stamp. */
 
   struct list *maxage_lsa;              /* List of MaxAge LSA for deletion. */
   int redistribute;                     /* Num of redistributed protocols. */
@@ -245,10 +263,12 @@ struct ospf
 #endif /* HAVE_OPAQUE_LSA */
   struct thread *t_maxage;              /* MaxAge LSA remover timer. */
   struct thread *t_maxage_walker;       /* MaxAge LSA checking timer. */
+  struct thread *t_graceful_shutdown;	/* Graceful/stub-router shutdown timer*/
 
   struct thread *t_write;
   struct thread *t_read;
   int fd;
+  struct stream *ibuf;
   struct list *oi_write_q;
   
   /* Distribute lists out of other route sources. */
@@ -342,6 +362,7 @@ struct ospf_area
   int shortcut_capability;              /* Other ABRs agree on S-bit */
   u_int32_t default_cost;               /* StubDefaultCost. */
   int auth_type;                        /* Authentication type. */
+  
 
   u_char NSSATranslatorRole;          /* NSSA configured role */
 #define OSPF_NSSA_ROLE_NEVER     0
@@ -356,7 +377,13 @@ struct ospf_area
 #define OSPF_TRANSIT_FALSE      0
 #define OSPF_TRANSIT_TRUE       1
   struct route_table *ranges;		/* Configured Area Ranges. */
-
+  
+  /* RFC3137 stub router state flags for area */
+  u_char stub_router_state;
+#define OSPF_AREA_ADMIN_STUB_ROUTED	(1 << 0) /* admin stub-router set */
+#define OSPF_AREA_IS_STUB_ROUTED	(1 << 1) /* stub-router active */
+#define OSPF_AREA_WAS_START_STUB_ROUTED	(1 << 2) /* startup SR was done */
+  
   /* Area related LSDBs[Type1-4]. */
   struct ospf_lsdb *lsdb;
 
@@ -406,6 +433,7 @@ struct ospf_area
 
   /* Threads. */
   struct thread *t_router_lsa_self;/* Self-originated router-LSA timer. */
+  struct thread *t_stub_router;    /* Stub-router timer */
 #ifdef HAVE_OPAQUE_LSA
   struct thread *t_opaque_lsa_self;	/* Type-10 Opaque-LSAs origin. */
 #endif /* HAVE_OPAQUE_LSA */
@@ -512,7 +540,6 @@ extern struct message ospf_ism_state_msg[];
 extern struct message ospf_nsm_state_msg[];
 extern struct message ospf_lsa_type_msg[];
 extern struct message ospf_link_state_id_type_msg[];
-extern struct message ospf_redistributed_proto[];
 extern struct message ospf_network_type_msg[];
 extern int ospf_ism_state_msg_max;
 extern int ospf_nsm_state_msg_max;
@@ -525,57 +552,65 @@ extern struct thread_master *master;
 extern int ospf_zlog;
 
 /* Prototypes. */
-struct ospf *ospf_lookup ();
-struct ospf *ospf_get ();
-void ospf_finish (struct ospf *);
-int ospf_router_id_update_timer (struct thread *);
-void ospf_router_id_update ();
-int ospf_network_match_iface (struct connected *, struct prefix *);
-int ospf_network_set (struct ospf *, struct prefix_ipv4 *, struct in_addr);
-int ospf_network_unset (struct ospf *, struct prefix_ipv4 *, struct in_addr);
-int ospf_area_stub_set (struct ospf *, struct in_addr);
-int ospf_area_stub_unset (struct ospf *, struct in_addr);
-int ospf_area_no_summary_set (struct ospf *, struct in_addr);
-int ospf_area_no_summary_unset (struct ospf *, struct in_addr);
-int ospf_area_nssa_set (struct ospf *, struct in_addr);
-int ospf_area_nssa_unset (struct ospf *, struct in_addr);
-int ospf_area_nssa_translator_role_set (struct ospf *, struct in_addr, int);
-int ospf_area_export_list_set (struct ospf *, struct ospf_area *, const char *);
-int ospf_area_export_list_unset (struct ospf *, struct ospf_area *);
-int ospf_area_import_list_set (struct ospf *, struct ospf_area *, const char *);
-int ospf_area_import_list_unset (struct ospf *, struct ospf_area *);
-int ospf_area_shortcut_set (struct ospf *, struct ospf_area *, int);
-int ospf_area_shortcut_unset (struct ospf *, struct ospf_area *);
-int ospf_timers_spf_set (struct ospf *, u_int32_t, u_int32_t);
-int ospf_timers_spf_unset (struct ospf *);
-int ospf_timers_refresh_set (struct ospf *, int);
-int ospf_timers_refresh_unset (struct ospf *);
-int ospf_nbr_nbma_set (struct ospf *, struct in_addr);
-int ospf_nbr_nbma_unset (struct ospf *, struct in_addr);
-int ospf_nbr_nbma_priority_set (struct ospf *, struct in_addr, u_char);
-int ospf_nbr_nbma_priority_unset (struct ospf *, struct in_addr);
-int ospf_nbr_nbma_poll_interval_set (struct ospf *, struct in_addr, unsigned int);
-int ospf_nbr_nbma_poll_interval_unset (struct ospf *, struct in_addr);
-void ospf_prefix_list_update (struct prefix_list *);
-void ospf_init ();
-void ospf_if_update (struct ospf *);
-void ospf_ls_upd_queue_empty (struct ospf_interface *);
-void ospf_terminate ();
-void ospf_nbr_nbma_if_update (struct ospf *, struct ospf_interface *);
-struct ospf_nbr_nbma *ospf_nbr_nbma_lookup (struct ospf *, struct in_addr);
-struct ospf_nbr_nbma *ospf_nbr_nbma_lookup_next (struct ospf *,
-						 struct in_addr *, int);
-int ospf_oi_count (struct interface *);
+extern const char *ospf_redist_string(u_int route_type);
+extern struct ospf *ospf_lookup (void);
+extern struct ospf *ospf_get (void);
+extern void ospf_finish (struct ospf *);
+extern int ospf_router_id_update_timer (struct thread *);
+extern void ospf_router_id_update (struct ospf *ospf);
+extern int ospf_network_match_iface (struct connected *, struct prefix *);
+extern int ospf_network_set (struct ospf *, struct prefix_ipv4 *,
+			     struct in_addr);
+extern int ospf_network_unset (struct ospf *, struct prefix_ipv4 *,
+			       struct in_addr);
+extern int ospf_area_stub_set (struct ospf *, struct in_addr);
+extern int ospf_area_stub_unset (struct ospf *, struct in_addr);
+extern int ospf_area_no_summary_set (struct ospf *, struct in_addr);
+extern int ospf_area_no_summary_unset (struct ospf *, struct in_addr);
+extern int ospf_area_nssa_set (struct ospf *, struct in_addr);
+extern int ospf_area_nssa_unset (struct ospf *, struct in_addr);
+extern int ospf_area_nssa_translator_role_set (struct ospf *, struct in_addr,
+					       int);
+extern int ospf_area_export_list_set (struct ospf *, struct ospf_area *,
+				      const char *);
+extern int ospf_area_export_list_unset (struct ospf *, struct ospf_area *);
+extern int ospf_area_import_list_set (struct ospf *, struct ospf_area *,
+				      const char *);
+extern int ospf_area_import_list_unset (struct ospf *, struct ospf_area *);
+extern int ospf_area_shortcut_set (struct ospf *, struct ospf_area *, int);
+extern int ospf_area_shortcut_unset (struct ospf *, struct ospf_area *);
+extern int ospf_timers_refresh_set (struct ospf *, int);
+extern int ospf_timers_refresh_unset (struct ospf *);
+extern int ospf_nbr_nbma_set (struct ospf *, struct in_addr);
+extern int ospf_nbr_nbma_unset (struct ospf *, struct in_addr);
+extern int ospf_nbr_nbma_priority_set (struct ospf *, struct in_addr, u_char);
+extern int ospf_nbr_nbma_priority_unset (struct ospf *, struct in_addr);
+extern int ospf_nbr_nbma_poll_interval_set (struct ospf *, struct in_addr,
+					    unsigned int);
+extern int ospf_nbr_nbma_poll_interval_unset (struct ospf *, struct in_addr);
+extern void ospf_prefix_list_update (struct prefix_list *);
+extern void ospf_init (void);
+extern void ospf_if_update (struct ospf *);
+extern void ospf_ls_upd_queue_empty (struct ospf_interface *);
+extern void ospf_terminate (void);
+extern void ospf_nbr_nbma_if_update (struct ospf *, struct ospf_interface *);
+extern struct ospf_nbr_nbma *ospf_nbr_nbma_lookup (struct ospf *,
+						   struct in_addr);
+extern struct ospf_nbr_nbma *ospf_nbr_nbma_lookup_next (struct ospf *,
+							struct in_addr *,
+							int);
+extern int ospf_oi_count (struct interface *);
 
-struct ospf_area *ospf_area_get (struct ospf *, struct in_addr, int);
-void ospf_area_check_free (struct ospf *, struct in_addr);
-struct ospf_area *ospf_area_lookup_by_area_id (struct ospf *, struct in_addr);
-void ospf_area_add_if (struct ospf_area *, struct ospf_interface *);
-void ospf_area_del_if (struct ospf_area *, struct ospf_interface *);
+extern struct ospf_area *ospf_area_get (struct ospf *, struct in_addr, int);
+extern void ospf_area_check_free (struct ospf *, struct in_addr);
+extern struct ospf_area *ospf_area_lookup_by_area_id (struct ospf *,
+						      struct in_addr);
+extern void ospf_area_add_if (struct ospf_area *, struct ospf_interface *);
+extern void ospf_area_del_if (struct ospf_area *, struct ospf_interface *);
 
-void ospf_route_map_init ();
-void ospf_snmp_init ();
+extern void ospf_route_map_init (void);
+extern void ospf_snmp_init (void);
 
-void ospf_master_init ();
+extern void ospf_master_init (void);
 
 #endif /* _ZEBRA_OSPFD_H */
