@@ -43,7 +43,7 @@
 
 
 /* Called when new interface is added. */
-int
+static int
 if_zebra_new_hook (struct interface *ifp)
 {
   struct zebra_if *zebra_if;
@@ -89,7 +89,7 @@ if_zebra_new_hook (struct interface *ifp)
 }
 
 /* Called when interface is deleted. */
-int
+static int
 if_zebra_delete_hook (struct interface *ifp)
 {
   struct zebra_if *zebra_if;
@@ -189,7 +189,7 @@ if_subnet_delete (struct interface *ifp, struct connected *ifc)
 
 /* Wake up configured address if it is not in current kernel
    address. */
-void
+static void
 if_addr_wakeup (struct interface *ifp)
 {
   struct listnode *node, *nnode;
@@ -209,6 +209,16 @@ if_addr_wakeup (struct interface *ifp)
 	    {
 	      if (! if_is_up (ifp))
 		{
+		  /* XXX: WTF is it trying to set flags here?
+		   * caller has just gotten a new interface, has been
+                   * handed the flags already. This code has no business
+                   * trying to override administrative status of the interface.
+                   * The only call path to here which doesn't originate from
+                   * kernel event is irdp - what on earth is it trying to do?
+                   *
+                   * further RUNNING is not a settable flag on any system
+                   * I (paulj) am aware of.
+                   */
 		  if_set_flags (ifp, IFF_UP | IFF_RUNNING);
 		  if_refresh (ifp);
 		}
@@ -236,6 +246,7 @@ if_addr_wakeup (struct interface *ifp)
 	    {
 	      if (! if_is_up (ifp))
 		{
+		  /* XXX: See long comment above */
 		  if_set_flags (ifp, IFF_UP | IFF_RUNNING);
 		  if_refresh (ifp);
 		}
@@ -290,14 +301,7 @@ if_add_update (struct interface *ifp)
     }
 }
 
-
-/* Handle an interface delete event
- * 
- * This function is only called  when support for
- * RTM_IFANNOUNCE or AF_NETLINK sockets (RTM_DELLINK message)
- * is available. It is not called on, eg, Solaris.
- */
-#if (defined(RTM_IFANNOUNCE) || defined(HAVE_NETLINK))
+/* Handle an interface delete event */
 void 
 if_delete_update (struct interface *ifp)
 {
@@ -405,7 +409,6 @@ if_delete_update (struct interface *ifp)
      interface deletion message. */
   ifp->ifindex = IFINDEX_INTERNAL;
 }
-#endif /* (defined(RTM_IFANNOUNCE) || defined(HAVE_NETLINK) */
 
 /* Interface is up. */
 void
@@ -490,7 +493,7 @@ if_refresh (struct interface *ifp)
 }
 
 /* Printout flag information into vty */
-void
+static void
 if_flag_dump_vty (struct vty *vty, unsigned long flag)
 {
   int separator = 0;
@@ -530,7 +533,7 @@ if_flag_dump_vty (struct vty *vty, unsigned long flag)
 }
 
 /* Output prefix string to vty. */
-int
+static int
 prefix_vty_out (struct vty *vty, struct prefix *p)
 {
   char str[INET6_ADDRSTRLEN];
@@ -541,7 +544,7 @@ prefix_vty_out (struct vty *vty, struct prefix *p)
 }
 
 /* Dump if address information to vty. */
-void
+static void
 connected_dump_vty (struct vty *vty, struct connected *connected)
 {
   struct prefix *p;
@@ -585,7 +588,7 @@ connected_dump_vty (struct vty *vty, struct connected *connected)
 
 #ifdef RTADV
 /* Dump interface ND information to vty. */
-void
+static void
 nd_dump_vty (struct vty *vty, struct interface *ifp)
 {
   struct zebra_if *zif;
@@ -630,7 +633,7 @@ nd_dump_vty (struct vty *vty, struct interface *ifp)
 #endif /* RTADV */
 
 /* Interface's information print out to vty interface. */
-void
+static void
 if_dump_vty (struct vty *vty, struct interface *ifp)
 {
 #ifdef HAVE_SOCKADDR_DL
@@ -813,7 +816,7 @@ if_dump_vty (struct vty *vty, struct interface *ifp)
 }
 
 /* Check supported address family. */
-int
+static int
 if_supported_family (int family)
 {
   if (family == AF_INET)
@@ -1141,7 +1144,7 @@ ALIAS (no_bandwidth_if,
        "Set bandwidth informational parameter\n"
        "Bandwidth in kilobits\n")
 
-int
+static int
 ip_address_install (struct vty *vty, struct interface *ifp,
 		    const char *addr_str, const char *peer_str,
 		    const char *label)
@@ -1158,7 +1161,7 @@ ip_address_install (struct vty *vty, struct interface *ifp,
       return CMD_WARNING;
     }
 
-  ifc = connected_check_ipv4 (ifp, (struct prefix *) &cp);
+  ifc = connected_check (ifp, (struct prefix *) &cp);
   if (! ifc)
     {
       ifc = connected_new ();
@@ -1180,7 +1183,7 @@ ip_address_install (struct vty *vty, struct interface *ifp,
 
       /* Label. */
       if (label)
-	ifc->label = strdup (label);
+	ifc->label = XSTRDUP (MTYPE_CONNECTED_LABEL, label);
 
       /* Add to linked list. */
       listnode_add (ifp->connected, ifc);
@@ -1226,7 +1229,7 @@ ip_address_install (struct vty *vty, struct interface *ifp,
   return CMD_SUCCESS;
 }
 
-int
+static int
 ip_address_uninstall (struct vty *vty, struct interface *ifp,
 		      const char *addr_str, const char *peer_str,
 		      const char *label)
@@ -1244,7 +1247,7 @@ ip_address_uninstall (struct vty *vty, struct interface *ifp,
     }
 
   /* Check current interface address. */
-  ifc = connected_check_ipv4 (ifp, (struct prefix *) &cp);
+  ifc = connected_check (ifp, (struct prefix *) &cp);
   if (! ifc)
     {
       vty_out (vty, "%% Can't find address%s", VTY_NEWLINE);
@@ -1337,7 +1340,7 @@ DEFUN (no_ip_address_label,
 #endif /* HAVE_NETLINK */
 
 #ifdef HAVE_IPV6
-int
+static int
 ipv6_address_install (struct vty *vty, struct interface *ifp,
 		      const char *addr_str, const char *peer_str,
 		      const char *label, int secondary)
@@ -1354,7 +1357,7 @@ ipv6_address_install (struct vty *vty, struct interface *ifp,
       return CMD_WARNING;
     }
 
-  ifc = connected_check_ipv6 (ifp, (struct prefix *) &cp);
+  ifc = connected_check (ifp, (struct prefix *) &cp);
   if (! ifc)
     {
       ifc = connected_new ();
@@ -1371,7 +1374,7 @@ ipv6_address_install (struct vty *vty, struct interface *ifp,
 
       /* Label. */
       if (label)
-	ifc->label = strdup (label);
+	ifc->label = XSTRDUP (MTYPE_CONNECTED_LABEL, label);
 
       /* Add to linked list. */
       listnode_add (ifp->connected, ifc);
@@ -1415,7 +1418,7 @@ ipv6_address_install (struct vty *vty, struct interface *ifp,
   return CMD_SUCCESS;
 }
 
-int
+static int
 ipv6_address_uninstall (struct vty *vty, struct interface *ifp,
 			const char *addr_str, const char *peer_str,
 			const char *label, int secondry)
@@ -1433,7 +1436,7 @@ ipv6_address_uninstall (struct vty *vty, struct interface *ifp,
     }
 
   /* Check current interface address. */
-  ifc = connected_check_ipv6 (ifp, (struct prefix *) &cp);
+  ifc = connected_check (ifp, (struct prefix *) &cp);
   if (! ifc)
     {
       vty_out (vty, "%% Can't find address%s", VTY_NEWLINE);
@@ -1497,7 +1500,7 @@ DEFUN (no_ipv6_address,
 }
 #endif /* HAVE_IPV6 */
 
-int
+static int
 if_config_write (struct vty *vty)
 {
   struct listnode *node;
@@ -1571,7 +1574,7 @@ if_config_write (struct vty *vty)
 
 /* Allocate and initialize interface vector. */
 void
-zebra_if_init ()
+zebra_if_init (void)
 {
   /* Initialize interface and new hook. */
   if_init ();
